@@ -3,11 +3,9 @@
 /* #undef VERSION */
 #undef HAVE_LIMITED_WRITEV
 
-/* Define if you want to indent the output code.  */
-#undef HAVE_INDENT
+/* Define to the command to indent the output code with, must read stdin.  */
+#undef INDENT_COMMAND
 
-/* Define if HAVE_INDENT is defined and you want to indent the output code in K&R style.  */
-#undef HAVE_INDENT_KR
 #undef HAVE_INET_ATON
 #undef ORBIT_SERIAL
 
