@@ -18,7 +18,7 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: util.c,v 1.106 1999/04/02 00:52:29 andrewtv Exp $
+    $Id: util.c,v 1.112 1999/06/09 16:22:16 andrewtv Exp $
 
 ***************************************************************************/
 #include <assert.h>
@@ -220,7 +220,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 		return -1;
 	}
 
-#ifndef _WIN32
+#ifndef NO_ACCESS
 	if (access (filename, R_OK))
 		return -1;
 #endif
@@ -276,11 +276,13 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	}
 	strcpy (tmpfilename, s);
 	strcat (tmpfilename, ".c");
+#ifndef NO_SYMLINK
 	if (symlink (linkto, tmpfilename) < 0) {
 		g_free (linkto);
 		g_free (tmpfilename);
 		return -1;
 	}
+#endif
 	g_free (linkto);
 
 	cmd_len = (strlen (tmpfilename) + strlen (cwd) +
@@ -297,7 +299,11 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 		    cwd, cpp_args ? cpp_args : "", tmpfilename);
 #endif
 
+#ifndef NO_POPEN
 	input = popen (cmd, "r");
+#else
+	input = fopen (cmd, "r");
+#endif
 	g_free (cmd);
 
 	if (input == NULL || ferror (input)) {
@@ -334,11 +340,16 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	g_hash_table_destroy (__IDL_structunion_ht);
 	__IDL_is_parsing = FALSE;
 	__IDL_lex_cleanup ();
+	__IDL_parser_reset ();
 	__IDL_real_filename = NULL;
 #ifndef HAVE_CPP_PIPE_STDIN
 	__IDL_tmp_filename = NULL;
 #endif
+#ifndef NO_POPEN
 	pclose (input);
+#else
+	fclose (input);
+#endif
 #ifndef HAVE_CPP_PIPE_STDIN
 	unlink (tmpfilename);
 	g_free (tmpfilename);
@@ -439,6 +450,7 @@ int IDL_parse_filename_with_input (const char *filename,
 	g_hash_table_destroy (__IDL_structunion_ht);
 	__IDL_is_parsing = FALSE;
 	__IDL_lex_cleanup ();
+	__IDL_parser_reset ();
 	__IDL_real_filename = NULL;
 	for (slist = __IDL_new_ident_comments; slist; slist = slist->next)
 		g_free (slist->data);
@@ -3052,7 +3064,6 @@ static gboolean IDL_emit_IDL_op_dcl_pre (IDL_tree_func_data *tfd, IDL_output_dat
 	IDL_emit_IDL_indent (tfd, data);
 	data->inline_props = TRUE;
 	IDL_emit_IDL_properties (IDL_OP_DCL (tfd->tree).ident, data);
-	if (IDL_OP_DCL (tfd->tree).f_noscript) dataf (data, "noscript" DELIM_SPACE);
 	if (IDL_OP_DCL (tfd->tree).f_oneway) dataf (data, "oneway" DELIM_SPACE);
 	if (IDL_OP_DCL (tfd->tree).op_type_spec) {
 		save_flag (idents, TRUE);
