@@ -1043,6 +1043,9 @@ iiop_connection_server_accept(GIOPConnection *connection)
     newcnx = GIOP_CONNECTION(iiop_connection_from_fd(newfd,
 						     IIOP_CONNECTION(connection)));
     GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
+    switch(IIOP_CONNECTION(connection)->icnxtype) {
+    case IIOP_USOCK: newcnx->is_auth = TRUE; break;
+    }
   }
 }
 
