@@ -22,6 +22,8 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <fcntl.h>
 
 #ifdef HAVE_POLL
 #  include <sys/poll.h>
@@ -173,9 +175,11 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
   struct iovec *curvec;
   int fd;
   GIOPConnection *cnx;
+  gint retval = -1;
 
   cnx = GIOP_MESSAGE_BUFFER(send_buffer)->connection;
-  g_return_val_if_fail(cnx->is_valid, -1);
+  if(!cnx->is_valid)
+    return -1;
 
   fd = GIOP_CONNECTION_GET_FD(cnx);
   nvecs = GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->len;
@@ -191,27 +195,51 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 	    sum);
   }
 #endif
-
   res = writev(fd, curvec, nvecs);
 
-  for(sum = res; sum < (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader)); ) {
+  sum = (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader));
+  if(res < sum) {
     if(res < 0) {
-      giop_main_handle_connection_exception(cnx);
-      return -1;
+      if(errno != EAGAIN) {
+	giop_main_handle_connection_exception(cnx);
+	goto out;
+      }
+
+      res = 0;
     }
 
-    for(i = t = 0; t < sum; i++)
-      t += curvec[i].iov_len;
-    curvec = &curvec[i-1];
-    curvec->iov_base = ((guchar *)curvec->iov_base) + (t - sum);
-    curvec->iov_len -= t - sum;
+    /* wrote 7, iovecs 3, 2, 2, 4:
+       0 + 3 !> 7
+       3 + 2 !> 7
+       5 + 2 !> 7
+     */
 
-    nvecs -= i - 1;
+    for(t = 0; ; curvec++, nvecs--, t += curvec->iov_len) {
+      if((t + curvec->iov_len) > res)
+	break;
+    }
+    if((res - t) > 0) {
+      curvec->iov_len -= (res - t);
+      curvec->iov_base += (res - t);
+    }
+
+    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NONBLOCK);
 
     res = writev(fd, curvec, nvecs);
+
+    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
+
+    if(res < sum) {
+	giop_main_handle_connection_exception(cnx);
+	goto out;
+    }
   }
 
-  return 0;
+  retval = 0;
+
+ out:
+
+  return retval;
 }
 
 static GIOPSendBuffer *
@@ -219,7 +247,8 @@ giop_send_buffer_use(GIOPConnection *connection)
 {
   GIOPSendBuffer *retval;
 
-  g_return_val_if_fail(connection->is_valid, NULL);
+  if(!connection->is_valid)
+    return NULL;
 
   GET_LOCK(sendbufferlist);
 
@@ -264,7 +293,8 @@ giop_send_reply_buffer_use(GIOPConnection *connection,
 
   send_buffer = giop_send_buffer_use(connection);
 
-  g_return_val_if_fail(send_buffer, NULL);
+  if(!send_buffer)
+    return NULL;
 
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REPLY;
 
@@ -309,18 +339,19 @@ giop_send_locate_reply_buffer_use(GIOPConnection *connection,
 			   GIOP_unsigned_long request_id,
 			   GIOPLocateStatusType locate_reply_status)
 {
-	GIOPSendBuffer *send_buffer;
+  GIOPSendBuffer *send_buffer;
 
-	send_buffer = giop_send_buffer_use(connection);
+  send_buffer = giop_send_buffer_use(connection);
 
-	g_return_val_if_fail(send_buffer, NULL);
+  if(!send_buffer)
+    return NULL;
 
-	GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_LOCATEREPLY;
+  GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_LOCATEREPLY;
 
-	APIA(&request_id, sizeof(request_id));
-	APIA(&locate_reply_status, sizeof(locate_reply_status));
+  APIA(&request_id, sizeof(request_id));
+  APIA(&locate_reply_status, sizeof(locate_reply_status));
 
-	return send_buffer;
+  return send_buffer;
 }
 
 GIOPSendBuffer *
@@ -344,9 +375,11 @@ giop_send_request_buffer_use(GIOPConnection *connection,
 #endif
 
   if (!connection)
-	  return NULL;
-  g_return_val_if_fail(object_key_vec, NULL);
-  g_return_val_if_fail(operation_vec, NULL);
+    return NULL;
+  if(!object_key_vec)
+    return NULL;
+  if(!operation_vec)
+    return NULL;
 
   ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 	      "Sending request %s id %d to %s\n",
@@ -355,7 +388,8 @@ giop_send_request_buffer_use(GIOPConnection *connection,
 
   send_buffer = giop_send_buffer_use(connection);
 
-  g_return_val_if_fail(send_buffer, NULL);
+  if (!send_buffer)
+    return NULL;
 
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REQUEST;
 
@@ -434,8 +468,8 @@ giop_send_locate_request_buffer_use(GIOPConnection *connection,
 
 	if (!connection)
 		return NULL;
-
-	g_return_val_if_fail(object_key_vec, NULL);
+	if (!object_key_vec)
+		return NULL;
 
 	ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 		"Sending locate request id %d to %s\n",
@@ -443,7 +477,8 @@ giop_send_locate_request_buffer_use(GIOPConnection *connection,
 
 	send_buffer = giop_send_buffer_use(connection);
 
-	g_return_val_if_fail(send_buffer, NULL);
+	if (!send_buffer)
+		return NULL;
 
 	GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_LOCATEREQUEST;
 
@@ -742,7 +777,8 @@ giop_recv_buffer_use(GIOPConnection *connection)
 {
   GIOPRecvBuffer *retval;
 
-  g_return_val_if_fail(connection->is_valid, NULL);
+  if(!connection || !connection->is_valid)
+    return NULL;
 
   GET_LOCK(recvbufferlist);
 
@@ -781,11 +817,9 @@ giop_recv_message_buffer_use(GIOPConnection *connection)
   int sysret;
   guint message_size;
   
-  if (!connection)
+  if(!connection || !connection->is_valid)
     return NULL;
 
-  g_return_val_if_fail(connection->is_valid, NULL);
-
   if(connection->incoming_msg)
     retval = connection->incoming_msg;
   else {
@@ -1093,15 +1127,16 @@ giop_recv_decode_message(GIOPRecvBuffer *buf)
 /* if(currptr+len > end || currptr + len < currptr) */
 
 /* This whole mess needs redoing. */
-
 #define CHECK_NEW_POS(buf, requested_increment) \
-g_return_val_if_fail( (( ((guchar*)GIOP_RECV_BUFFER(buf)->cur) \
+if(!( (( ((guchar*)GIOP_RECV_BUFFER(buf)->cur) \
 		        + (requested_increment) ) \
 		       <= ( ((guchar *)GIOP_RECV_BUFFER(buf)->message_body) \
 			   + GIOP_MESSAGE_BUFFER(buf)->message_header.message_size)) \
 		      && ( ( ((guchar*)GIOP_RECV_BUFFER(buf)->cur) \
 			    + (requested_increment) ) \
-			  >= ((guchar*)GIOP_RECV_BUFFER(buf)->cur) ), -1)
+			  >= ((guchar*)GIOP_RECV_BUFFER(buf)->cur) ))) goto out;
+
+#define NEW_POS_OUT out: return -1
 
 #define SAFE_ALIGN_ADDRESS(buf, amt) G_STMT_START { \
 guchar *newval; \
@@ -1198,6 +1233,8 @@ solve this problem */
 #endif
 
   return 0;
+
+  NEW_POS_OUT;
 }
 
 static gint
@@ -1215,6 +1252,7 @@ giop_recv_locate_reply_decode_message(GIOPRecvBuffer *buf)
     }
 
   return 0;
+  NEW_POS_OUT;
 }
 
 static gint
@@ -1326,6 +1364,8 @@ giop_recv_request_decode_message(GIOPRecvBuffer *buf)
 #endif
 
   return 0;
+
+  NEW_POS_OUT;
 }
 
 static gint
@@ -1353,6 +1393,8 @@ giop_recv_locate_request_decode_message(GIOPRecvBuffer *buf)
     }
 
   return 0;
+
+  NEW_POS_OUT;
 }
 
 gboolean
