@@ -1,8 +1,8 @@
 #include "iiop-endian.h"
 
-void byteswap(guchar *outdata,
-	      const guchar *data,
-	      gulong datalen)
+void iiop_byteswap(guchar *outdata,
+		   const guchar *data,
+		   gulong datalen)
 {
   const guchar *source_ptr = data;
   guchar *dest_ptr = (guchar *)outdata + datalen - 1;
