@@ -3,12 +3,12 @@
 
 #include <glib.h>
 
-#define DEBUG 1
+#define ORBIT_DEBUG 1
 
-#ifdef DEBUG
-#define DEBUG_NOTE(x) (x)
+#ifdef ORBIT_DEBUG
+#define ORBIT_DEBUG_NOTE(x) (x)
 #else
-#define DEBUG_NOTE(x)
+#define ORBIT_DEBUG_NOTE(x)
 #endif
 
 
@@ -17,8 +17,15 @@
 #define BitClr(f, bit) ((f) &= ~(1<<(bit)))
 /* Align an address upward to a boundary, expressed as a number of bytes.
    E.g. align to an 8-byte boundary with argument of 8.  */
+
+/*
+ *   (this + boundary - 1)
+ *          &
+ *    ~(boundary - 1)
+ */
+
 #define ALIGN_ADDRESS(this, boundary) \
-  ((gpointer)((( ((unsigned long)(this)) + (((unsigned long)boundary) -1)) & (~(((unsigned long)boundary)-1)))))
+  ((gpointer)((( ((unsigned long)(this)) + (((unsigned long)(boundary)) -1)) & (~(((unsigned long)(boundary))-1)))))
 
 
 #include "thread-safety.h"
