@@ -547,7 +547,7 @@ CORBA_boolean CDR_get_string_static(CDR_Codec *codec,
 	if((codec->rptr + len) > codec->buf_len)
 		return CORBA_FALSE;
 
-	*str = codec->buffer + codec->rptr;
+	*str = ((CORBA_char *)codec->buffer) + codec->rptr;
 
 	codec->rptr += len;
 
