@@ -307,7 +307,7 @@ ORBit_send_user_exception(GIOPSendBuffer *send_buffer,
 			break;
 	}
 
-	if(user_exceptions[i].tc == CORBA_OBJECT_NIL || !ev->_params) {
+	if(user_exceptions[i].tc == CORBA_OBJECT_NIL) {
 		CORBA_Environment fakeev;
 		CORBA_exception_init(&fakeev);
 		CORBA_exception_set_system(&fakeev, ex_CORBA_UNKNOWN,
@@ -317,7 +317,8 @@ ORBit_send_user_exception(GIOPSendBuffer *send_buffer,
 	} else {
 		ENCODER_CALL(CORBA_char, ev->_repo_id);
 
-		user_exceptions[i].marshal(send_buffer, ev);
+		if(user_exceptions[i].marshal && ev->_params)
+			user_exceptions[i].marshal(send_buffer, ev);
 	}
 }
 
