@@ -67,13 +67,19 @@ static const ORBit_RootObject_Interface CORBA_POA_epv =
 guint
 g_sequence_octet_hash(CORBA_sequence_octet *so)
 {
-	guint retval = 0, i, n;
+  const char *s = (char*)so->_buffer;
+  const char *p, *e = ((char *)so->_buffer) + so->_length;
+  guint h=0, g;
 
-	n = so->_length/4;
-	for(i = 0; i < n; i++)
-		retval ^= ((CORBA_unsigned_long *)so->_buffer)[i];
+  for(p = s; p < e; p ++) {
+    h = ( h << 4 ) + *p;
+    if ( ( g = h & 0xf0000000 ) ) {
+      h = h ^ (g >> 24);
+      h = h ^ g;
+    }
+  }
 
-	return retval;
+  return h;
 }
 
 gint
