@@ -179,7 +179,7 @@ static void tc_enc(CORBA_TypeCode tc,
 	CDR_Codec encaps_d;
 	CDR_Codec* encaps = &encaps_d;
 
-	g_assert(tc->kind < CORBA_tk_last);
+	g_assert(CLAMP(0, tc->kind, CORBA_tk_last) == tc->kind);
 
 	for(l=ctx->prior_tcs;l;l=l->next){
 		TCRecursionNode* node=l->data;
@@ -282,6 +282,8 @@ static void tc_dec(CORBA_TypeCode* t, CDR_Codec* c, TCDecodeContext* ctx)
 
 	CDR_get_ulong(c, &kind);
 
+	g_assert(CLAMP(0, kind, CORBA_tk_last) == kind);
+
 	if(kind==CORBA_tk_recursive){
 		CORBA_long offset;
 		GSList* l;
